#!/usr/bin/perl

##################################################################################################################
use Test::More tests => 210;
##################################################################################################################
# basic object methods loads and executes
use Integrator::Module::Build;

# inherited stuff
can_ok ('Module::Build::Base', 'ACTION_build'		);
can_ok ('Module::Build::Base', 'ACTION_clean'		);
can_ok ('Module::Build::Base', 'ACTION_code'		);
can_ok ('Module::Build::Base', 'ACTION_config_data'	);
can_ok ('Module::Build::Base', 'ACTION_diff'		);
can_ok ('Module::Build::Base', 'ACTION_dist'		);
can_ok ('Module::Build::Base', 'ACTION_distcheck'	);
can_ok ('Module::Build::Base', 'ACTION_distclean'	);
can_ok ('Module::Build::Base', 'ACTION_distdir'		);
can_ok ('Module::Build::Base', 'ACTION_distmeta'		);
can_ok ('Module::Build::Base', 'ACTION_distsign'		);
can_ok ('Module::Build::Base', 'ACTION_disttest'		);
can_ok ('Module::Build::Base', 'ACTION_docs'		);
can_ok ('Module::Build::Base', 'ACTION_fakeinstall'	);
can_ok ('Module::Build::Base', 'ACTION_help'		);
can_ok ('Module::Build::Base', 'ACTION_html'		);
can_ok ('Module::Build::Base', 'ACTION_install'		);
can_ok ('Module::Build::Base', 'ACTION_manifest'		);
can_ok ('Module::Build::Base', 'ACTION_manpages'		);
can_ok ('Module::Build::Base', 'ACTION_ppd'		);
can_ok ('Module::Build::Base', 'ACTION_ppmdist'		);
can_ok ('Module::Build::Base', 'ACTION_prereq_report'		);
can_ok ('Module::Build::Base', 'ACTION_pure_install'		);
can_ok ('Module::Build::Base', 'ACTION_realclean'		);
can_ok ('Module::Build::Base', 'ACTION_skipcheck'		);
can_ok ('Module::Build::Base', 'ACTION_test'		);
can_ok ('Module::Build::Base', 'ACTION_testcover'		);
can_ok ('Module::Build::Base', 'ACTION_testdb'		);
can_ok ('Module::Build::Base', 'ACTION_testpod'		);
can_ok ('Module::Build::Base', 'ACTION_testpodcoverage'		);
can_ok ('Module::Build::Base', 'ACTION_versioninstall'		);
can_ok ('Module::Build::Base', 'PL_files'		);
can_ok ('Module::Build::Base', 'add_build_element'		);
can_ok ('Module::Build::Base', 'add_property'		);
can_ok ('Module::Build::Base', 'add_to_cleanup'		);
can_ok ('Module::Build::Base', 'allow_mb_mismatch'		);
can_ok ('Module::Build::Base', 'args'		);
can_ok ('Module::Build::Base', 'array_properties'		);
can_ok ('Module::Build::Base', 'auto_features'		);
can_ok ('Module::Build::Base', 'autosplit'		);
can_ok ('Module::Build::Base', 'autosplit_file'		);
can_ok ('Module::Build::Base', 'base_dir'		);
can_ok ('Module::Build::Base', 'bindoc_dirs'		);
can_ok ('Module::Build::Base', 'blib'		);
can_ok ('Module::Build::Base', 'build_bat'		);
can_ok ('Module::Build::Base', 'build_class'		);
can_ok ('Module::Build::Base', 'build_elements'		);
can_ok ('Module::Build::Base', 'build_requires'		);
can_ok ('Module::Build::Base', 'build_script'		);
can_ok ('Module::Build::Base', 'c_source'		);
can_ok ('Module::Build::Base', 'carp'		);
can_ok ('Module::Build::Base', 'check_autofeatures'		);
can_ok ('Module::Build::Base', 'check_installed_status'		);
can_ok ('Module::Build::Base', 'check_installed_version'		);
can_ok ('Module::Build::Base', 'check_manifest'		);
can_ok ('Module::Build::Base', 'check_prereq'		);
can_ok ('Module::Build::Base', 'cleanup'		);
can_ok ('Module::Build::Base', 'compare_versions'		);
can_ok ('Module::Build::Base', 'compile_c'		);
can_ok ('Module::Build::Base', 'compile_xs'		);
can_ok ('Module::Build::Base', 'confess'		);
can_ok ('Module::Build::Base', 'config'		);
can_ok ('Module::Build::Base', 'config_data'		);
can_ok ('Module::Build::Base', 'config_dir'		);
can_ok ('Module::Build::Base', 'config_file'		);
can_ok ('Module::Build::Base', 'conflicts'		);
can_ok ('Module::Build::Base', 'contains_pod'		);
can_ok ('Module::Build::Base', 'copy_if_modified'		);
can_ok ('Module::Build::Base', 'create_build_script'		);
can_ok ('Module::Build::Base', 'create_makefile_pl'		);
can_ok ('Module::Build::Base', 'create_packlist'		);
can_ok ('Module::Build::Base', 'create_readme'		);
can_ok ('Module::Build::Base', 'croak'		);
can_ok ('Module::Build::Base', 'cull_args'		);
can_ok ('Module::Build::Base', 'cull_options'		);
can_ok ('Module::Build::Base', 'current'		);
can_ok ('Module::Build::Base', 'current_action'		);
can_ok ('Module::Build::Base', 'cwd'		);
can_ok ('Module::Build::Base', 'debugger'		);
can_ok ('Module::Build::Base', 'delete_filetree'		);
can_ok ('Module::Build::Base', 'depends_on'		);
can_ok ('Module::Build::Base', 'destdir'		);
can_ok ('Module::Build::Base', 'dir_contains'		);
can_ok ('Module::Build::Base', 'dispatch'		);
can_ok ('Module::Build::Base', 'dist_abstract'		);
can_ok ('Module::Build::Base', 'dist_author'		);
can_ok ('Module::Build::Base', 'dist_dir'		);
can_ok ('Module::Build::Base', 'dist_name'		);
can_ok ('Module::Build::Base', 'dist_version'		);
can_ok ('Module::Build::Base', 'dist_version_from'		);
can_ok ('Module::Build::Base', 'do_create_makefile_pl'		);
can_ok ('Module::Build::Base', 'do_create_metafile'		);
can_ok ('Module::Build::Base', 'do_create_readme'		);
can_ok ('Module::Build::Base', 'do_system'		);
can_ok ('Module::Build::Base', 'expand_test_dir'		);
can_ok ('Module::Build::Base', 'extra_compiler_flags'		);
can_ok ('Module::Build::Base', 'extra_linker_flags'		);
can_ok ('Module::Build::Base', 'feature'		);
can_ok ('Module::Build::Base', 'features'		);
can_ok ('Module::Build::Base', 'find_PL_files'		);
can_ok ('Module::Build::Base', 'find_dist_packages'		);
can_ok ('Module::Build::Base', 'find_module_by_name'		);
can_ok ('Module::Build::Base', 'find_perl_interpreter'		);
can_ok ('Module::Build::Base', 'find_pm_files'		);
can_ok ('Module::Build::Base', 'find_pod_files'		);
can_ok ('Module::Build::Base', 'find_script_files'		);
can_ok ('Module::Build::Base', 'find_test_files'		);
can_ok ('Module::Build::Base', 'find_xs_files'		);
can_ok ('Module::Build::Base', 'fix_shebang_line'		);
can_ok ('Module::Build::Base', 'get_action_docs'		);
can_ok ('Module::Build::Base', 'get_options'		);
can_ok ('Module::Build::Base', 'has_config_data'		);
can_ok ('Module::Build::Base', 'hash_properties'		);
can_ok ('Module::Build::Base', 'have_c_compiler'		);
can_ok ('Module::Build::Base', 'have_forkpipe'		);
can_ok ('Module::Build::Base', 'html_css'		);
can_ok ('Module::Build::Base', 'htmlify_pods'		);
can_ok ('Module::Build::Base', 'include_dirs'		);
can_ok ('Module::Build::Base', 'install_base'		);
can_ok ('Module::Build::Base', 'install_base_relpaths'		);
can_ok ('Module::Build::Base', 'install_destination'		);
can_ok ('Module::Build::Base', 'install_map'		);
can_ok ('Module::Build::Base', 'install_path'		);
can_ok ('Module::Build::Base', 'install_sets'		);
can_ok ('Module::Build::Base', 'install_types'		);
can_ok ('Module::Build::Base', 'installdirs'		);
can_ok ('Module::Build::Base', 'invoked_action'		);
can_ok ('Module::Build::Base', 'known_actions'		);
can_ok ('Module::Build::Base', 'libdoc_dirs'		);
can_ok ('Module::Build::Base', 'license'		);
can_ok ('Module::Build::Base', 'link_c'		);
can_ok ('Module::Build::Base', 'localize_dir_path'		);
can_ok ('Module::Build::Base', 'localize_file_path'		);
can_ok ('Module::Build::Base', 'log_info'		);
can_ok ('Module::Build::Base', 'log_verbose'		);
can_ok ('Module::Build::Base', 'log_warn'		);
can_ok ('Module::Build::Base', 'magic_number'		);
can_ok ('Module::Build::Base', 'make_executable'		);
can_ok ('Module::Build::Base', 'man1page_name'		);
can_ok ('Module::Build::Base', 'man3page_name'		);
can_ok ('Module::Build::Base', 'manify_bin_pods'		);
can_ok ('Module::Build::Base', 'manify_lib_pods'		);
can_ok ('Module::Build::Base', 'manpage_separator'		);
can_ok ('Module::Build::Base', 'mb_parents'		);
can_ok ('Module::Build::Base', 'mb_version'		);
can_ok ('Module::Build::Base', 'merge_args'		);
can_ok ('Module::Build::Base', 'merge_modulebuildrc'		);
can_ok ('Module::Build::Base', 'meta_add'		);
can_ok ('Module::Build::Base', 'meta_merge'		);
can_ok ('Module::Build::Base', 'metafile'		);
can_ok ('Module::Build::Base', 'module_name'		);
can_ok ('Module::Build::Base', 'new'		);
can_ok ('Module::Build::Base', 'new_from_context'		);
can_ok ('Module::Build::Base', 'notes'		);
can_ok ('Module::Build::Base', 'orig_dir'		);
can_ok ('Module::Build::Base', 'original_prefix'		);
can_ok ('Module::Build::Base', 'perl'		);
can_ok ('Module::Build::Base', 'perl_version'		);
can_ok ('Module::Build::Base', 'perl_version_to_float'		);
can_ok ('Module::Build::Base', 'pm_files'		);
can_ok ('Module::Build::Base', 'pod_files'		);
can_ok ('Module::Build::Base', 'pollute'		);
can_ok ('Module::Build::Base', 'ppm_name'		);
can_ok ('Module::Build::Base', 'prefix'		);
can_ok ('Module::Build::Base', 'prefix_relative'		);
can_ok ('Module::Build::Base', 'prefix_relpaths'		);
can_ok ('Module::Build::Base', 'prepare_metadata'		);
can_ok ('Module::Build::Base', 'prereq_action_types'		);
can_ok ('Module::Build::Base', 'prereq_failures'		);
can_ok ('Module::Build::Base', 'prereq_report'		);
can_ok ('Module::Build::Base', 'print_build_script'		);
can_ok ('Module::Build::Base', 'process_PL_files'		);
can_ok ('Module::Build::Base', 'process_files_by_extension'		);
can_ok ('Module::Build::Base', 'process_pm_files'		);
can_ok ('Module::Build::Base', 'process_pod_files'		);
can_ok ('Module::Build::Base', 'process_script_files'		);
can_ok ('Module::Build::Base', 'process_support_files'		);
can_ok ('Module::Build::Base', 'process_xs'		);
can_ok ('Module::Build::Base', 'process_xs_files'		);
can_ok ('Module::Build::Base', 'prompt'		);
can_ok ('Module::Build::Base', 'quiet'		);
can_ok ('Module::Build::Base', 'read_args'		);
can_ok ('Module::Build::Base', 'read_modulebuildrc'		);
can_ok ('Module::Build::Base', 'recommends'		);
can_ok ('Module::Build::Base', 'recurse_into'		);
can_ok ('Module::Build::Base', 'recursive_test_files'		);
can_ok ('Module::Build::Base', 'requires'		);
can_ok ('Module::Build::Base', 'resume'		);
can_ok ('Module::Build::Base', 'rscan_dir'		);
can_ok ('Module::Build::Base', 'run_perl_command'		);
can_ok ('Module::Build::Base', 'run_perl_script'		);
can_ok ('Module::Build::Base', 'runtime_params'		);
can_ok ('Module::Build::Base', 'script_files'		);
can_ok ('Module::Build::Base', 'scripts'		);
can_ok ('Module::Build::Base', 'split_like_shell'		);
can_ok ('Module::Build::Base', 'subclass'		);
can_ok ('Module::Build::Base', 'super_classes'		);
can_ok ('Module::Build::Base', 'test_files'		);
can_ok ('Module::Build::Base', 'unparse_args'		);
can_ok ('Module::Build::Base', 'up_to_date'		);
can_ok ('Module::Build::Base', 'use_rcfile'		);
can_ok ('Module::Build::Base', 'valid_licenses'		);
can_ok ('Module::Build::Base', 'valid_properties'		);
can_ok ('Module::Build::Base', 'valid_properties_defaults'		);
can_ok ('Module::Build::Base', 'valid_property'		);
can_ok ('Module::Build::Base', 'verbose'		);
can_ok ('Module::Build::Base', 'version_from_file'		);
can_ok ('Module::Build::Base', 'write_metafile'		);
can_ok ('Module::Build::Base', 'xs_files'		);
can_ok ('Module::Build::Base', 'y_n'		);
#can_ok ('Module::Build::Platform', 'Unix::is_executable'		);
#can_ok ('Module::Build::Platform', 'Unix::make_tarball'		);
#can_ok ('Module::Build::os_type'		);
